rm(list=ls(all=TRUE))
library(car); library(effects); library(MASS); library(rms); library(rpart) # you may have to install these libraries first with install.packages("car") etc.

### Section 2
# load data
raw.data <- read.table(file=file.choose(), header=TRUE, sep="\t") # load the data by choosing this file: <04_data.csv>
str(raw.data); summary(raw.data) # inspect the structure of the data in the file
attach(raw.data) # make the column names of the data available as variable names
dd <- datadist(raw.data); options(datadist="dd") # prepare R for using the function lrm for logistic regression modeling in Section 3



### Section 2.1: chi-square test
contig.table <- table(ANIM_POSSOR, GENITIVE, SPEAKER) # generate a first type of overview table
contig.table # show that table in the console

ftable(table(SPEAKER, ANIM_POSSOR, GENITIVE)) # generate a second type of overview table

learners <- chisq.test(contig.table[,,"learner"], correct=FALSE) # chi-squared test for the learners
learners # show the result of that chi-squared test in the console
natives <- chisq.test(contig.table[,,"native"], correct=FALSE) # chi-squared test for the native speakers
natives # show the result of that chi-squared test in the console

learners$expected #  # expected frequencies for the learners
natives$expected # expected frequencies for the native speakers

round(learners$residuals, 2) # Pearson residuals for the learners
round(natives$residuals, 2) # Pearson residuals for the native speakers

par(mfrow=c(1,2)) # create two plotting panels
   assocplot(t(contig.table[,,"learner"]), col=c("black", "grey")) # association plot for the learners
   assocplot(t(contig.table[,,"native"]), col=c("black", "grey")) # association plot for the native speakers
par(mfrow=c(1,1)) # restore default paneling of window [800x600]

sqrt(learners$statistic/(sum(learners$observed)*min(dim(learners$observed)-1))) # Cramer's V for the learners
sqrt(natives$statistic/(sum(natives$observed)*min(dim(natives$observed)-1))) # Cramer's V for the native speakers



### Section 2.2: U-test
tapply(LEN_POSSED, GENITIVE, mean) # compute the means of LEN_POSSED for each level of GENITIVE
round(tapply(LEN_POSSED, list(GENITIVE, SPEAKER), mean), 2) # compute the means of LEN_POSSED for the combination of each level of GENITIVE and each level of SPEAKER

round(tapply(LEN_POSSED, list(GENITIVE, SPEAKER), sd), 2) # compute the standard deviations of LEN_POSSED for the combination of each level of GENITIVE and each level of SPEAKER

par(mfcol=c(2, 2)) # create four plotting panels
   hist(LEN_POSSED[GENITIVE=="of" & SPEAKER=="learner"], xlim=c(0,20), ylim=c(0,100), breaks=seq(0,20,2),
        main="of-genitives of learners", xlab="Lengths of possesseds", ylab="Frequency")
   hist(LEN_POSSED[GENITIVE=="s" & SPEAKER=="learner"], xlim=c(0,20), ylim=c(0,100), breaks=seq(0,20,2),
        main="s-genitives of learners", xlab="Lengths of possesseds", ylab="Frequency")
   hist(LEN_POSSED[GENITIVE=="of" & SPEAKER=="native"], xlim=c(0,20), ylim=c(0,100), breaks=seq(0,20,2),
        main="of-genitives of native speakers", xlab="Lengths of possesseds", ylab="Frequency")
   hist(LEN_POSSED[GENITIVE=="s" & SPEAKER=="native"], xlim=c(0,20), ylim=c(0,100), breaks=seq(0,20,2),
        main="s-genitives of native speakers", xlab="Lengths of possesseds", ylab="Frequency")
par(mfcol=c(1, 1)) # restore default paneling of window [700x500]

# sapply(tapply(LEN_POSSED, list(GENITIVE, SPEAKER), shapiro.test), "[", 2) # Shapiro-Wilk tests

tapply(LEN_POSSED, list(GENITIVE, SPEAKER), median) # compute the medians of LEN_POSSED for the combination of each level of GENITIVE and each level of SPEAKER
tapply(LEN_POSSED, list(GENITIVE, SPEAKER), IQR) # compute the interquartile ranges of LEN_POSSED for the combination of each level of GENITIVE and each level of SPEAKER

wilcox.test(LEN_POSSED ~ GENITIVE, subset=SPEAKER=="learner", exact=FALSE, correct=FALSE) # compute a U-test to compare the medians of LEN_POSSED for each level of GENITIVE but only for the learners
wilcox.test(LEN_POSSED ~ GENITIVE, subset=SPEAKER=="native", exact=FALSE, correct=FALSE) # compute a U-test to compare the medians of LEN_POSSED for each level of GENITIVE but only for the native speakers

boxplot(LEN_POSSED ~ GENITIVE + SPEAKER, notch=TRUE, xlab="Genitive x Speaker", ylab="Length of possessed") # generate a boxplot
   abline(h=median(LEN_POSSED), lty=2) # draw a horizontal line at y=median length of ALL possesseds



### Section 3.1: logistic regression with two categorical independent variables
model.01 <- lrm(GENITIVE ~ ANIM_POSSOR * SPEAKER, x=TRUE, y=TRUE); model.01 # fit and show the first regression model
model.02 <- lrm(GENITIVE ~ ANIM_POSSOR + SPEAKER, x=TRUE, y=TRUE); model.02 # fit and show the second regression model

# determine the nature of the effect(s) numerically
predictions <- expand.grid(ANIM_POSSOR=levels(ANIM_POSSOR), SPEAKER=levels(SPEAKER)) # generate a data frame with all combinations of all variables
predictions["PREDICTIONS"] <- predict(model.02, newdata=predictions, type="fitted.ind") # compute the predicted probabilities of s-genitives for these four combinations
predictions # show the predictions

predictions.tab <- matrix(predictions$PREDICTIONS, ncol=2, dimnames=list(levels(ANIM_POSSOR), levels(SPEAKER))); predictions.tab # change the predictions into a nicer tabular format and show it

par(mfrow=c(1, 2)) # create panels
   temp <- barplot(predictions.tab, beside=TRUE, ylim=c(0, 1), col=gray(1/3 * 1:2), xlab="Speaker", ylab="Predicted probability of s-genitives") 
      legend(mean(temp), 0.95, fill=gray(1/3 * 1:2), legend=levels(ANIM_POSSOR), xjust=0.5, yjust=0.5, ncol=2)
      abline(h=0.5, lty=2); text(mean(temp), 0.5, c("of", "s"), pos=(c(1, 3)), font=3)
   temp <- barplot(t(predictions.tab), beside=TRUE, ylim=c(0, 1), col=gray(1/3 * 1:2), xlab="Animacy of possessor", ylab="Predicted probability of s-genitives")
      legend(mean(temp), 0.95, fill=gray(1/3 * 1:2), legend=levels(SPEAKER), xjust=0.5, yjust=0.5, ncol=2)
      abline(h=0.5, lty=2); text(mean(temp), 0.5, c("of", "s"), pos=(c(1, 3)), font=3)
par(mfrow=c(1, 1)) # restore default paneling of window [800x600]

# compute the individual predictions of the model
predictions.num <- predict(model.02, type="fitted.ind") # compute the predicted probability of s-genitives for each case in the data
predictions.cat <- ifelse(predictions.num>=0.5, "s", "of") # convert the predicted probability of s-genitives into a categorical prediction: of or s

# determine the classification accuracy
table(GENITIVE, predictions.cat) # tabulate the predictions with the observed genitive choices
sum(diag(table(GENITIVE, predictions.cat)))/length(predictions.cat) # check how often the prediction is correct



### Section 3.2: logistic regression with a categorical and an interval-scaled independent variable
model.01 <- lrm(GENITIVE ~ LEN_POSSED* SPEAKER, x=TRUE, y=TRUE); model.01

# determine the nature of the effect(s) numerically
predictions <- expand.grid(LEN_POSSED=1:max(LEN_POSSED), SPEAKER=levels(SPEAKER)) # generate a data frame with all combinations of all variables
predictions["PREDICTIONS"] <- predict(model.01, newdata=predictions, type="fitted.ind") # compute the predicted probabilities of s-genitives for these four combinations
predictions # show the predictions

# change the predictions into a nicer tabular format and show it
predictions.tab <- predictions$PREDICTIONS; dim(predictions.tab) <- c(17, 2)
rownames(predictions.tab) <- unique(predictions$LEN_POSSED)
colnames(predictions.tab) <- unique(predictions$SPEAKER)
predictions.tab

# determine the nature of the effect(s) graphically
plot(predictions$PREDICTIONS ~ predictions$LEN_POSSED, ylim=c(0, 1), pch=substr(predictions$SPEAKER, 1, 1), xlab="Length of the possessed", ylab="Predicted probability of s-genitive") # plot the predicted probabilities for all LEN_POSSED for both speaker groups
   grid(); abline(h=0.5, lty=2) # add a grid and a line at y=0.5
   text(8.5, 0.5, c("of", "s"), pos=(c(1, 3)), font=3) # add a small legend 

# compute the individual predictions of the model
predictions.num <- predict(model.01, type="fitted.ind") # compute the predicted probability of s-genitives for each case in the data
predictions.cat <- ifelse(predictions.num>=0.5, "s", "of") # convert the predicted probability of s-genitives into a categorical prediction: of or s

# determine the classification accuracy
table(GENITIVE, predictions.cat) # tabulate the predictions with the observed genitive choices
sum(diag(table(GENITIVE, predictions.cat)))/length(predictions.cat) # check how often the prediction is correct

# all variables together
model.01 <- lrm(GENITIVE ~ ANIM_POSSOR * LEN_POSSED * SPEAKER, x=TRUE, y=TRUE); model.01
model.02 <- lrm(GENITIVE ~ (ANIM_POSSOR + LEN_POSSED + SPEAKER)^2, x=TRUE, y=TRUE); model.02
model.03 <- lrm(GENITIVE ~ (ANIM_POSSOR + LEN_POSSED + SPEAKER)^2 - ANIM_POSSOR:SPEAKER, x=TRUE, y=TRUE); model.03
model.04 <- lrm(GENITIVE ~ (ANIM_POSSOR + LEN_POSSED + SPEAKER)^2 - ANIM_POSSOR:SPEAKER - ANIM_POSSOR:LEN_POSSED, x=TRUE, y=TRUE); model.04

# compute the individual predictions of the model
predictions.num <- predict(model.04, type="fitted.ind")
predictions.cat <- ifelse(predictions.num>=0.5, "s", "of")

# determine the classification accuracy
table(GENITIVE, predictions.cat)
sum(diag(table(GENITIVE, predictions.cat)))/length(predictions.cat)

# determine the nature of the effect(s) graphically
plot(allEffects(glm(formula(model.04), family=binomial)), rescale.axis=FALSE, ylim=c(0, 1), ylab="Predicted probability of s-genitive")



### Section 4.1: Conditional inference trees: an alternative to regressions
library(party)
plot(ctree(GENITIVE ~ ANIM_POSSOR + SPEAKER))
